"
 COPYRIGHT (c) 2010 by Jan Vrany, SWING Research Group. CTU in Prague
	      All Rights Reserved

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the 'Software'), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
"
"{ Package: 'stx:libbasic2' }"

BackgroundJob subclass:#BackgroundQueueProcessingJob
	instanceVariableNames:'queue queueAccessLock'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Support'
!

!BackgroundQueueProcessingJob class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2010 by Jan Vrany, SWING Research Group. CTU in Prague
	      All Rights Reserved

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the 'Software'), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
"
!

documentation
"
    A BackgroundQueueProcessingJob is a specialized form
    of a BackgroundJob for background processing of a queue.
    Each item in the queue (added by sending #add: item) 
    is processed (using the instance variable job).

    Implementation notes:
    The thread is running only if there is at least one item
    to process. When the queue is empty, the thread teminates.
    It is started again when a new item is added to the queue.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]
        queue       <OrderedCollection>     the queue of items to be procesed
        queueAccessLock<RecursionLock>      a lock used to synchronize
                                            access to the queue

    [class variables:]

    [see also:]
        BackgroundJob

"
!

examples
"
    | job text |
    job :=  BackgroundQueueProcessingJob named: 'example job' on:[:text|
                Delay waitForSeconds: 3.
                Transcript showCR:'One guy said: ', text
            ].
    job add:'Hello world'. 
    Delay waitForSeconds: 5.
    job add:'Ahoj Svete!!'.
    job restart.
    Delay waitForSeconds: 1.
    job add:'Haya, looks like proper queue, you should see all greetings'.

"
! !

!BackgroundQueueProcessingJob methodsFor:'adding & removing'!

add: object
    "includes: is not synchronized, but should not harm"

    queueAccessLock critical:[
        (queue includes: object)
            ifFalse:[queue add: object].
        self start
    ].

    "Created: / 28-04-2011 / 20:40:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 18-05-2011 / 23:30:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 03-08-2011 / 16:43:01 / cg"
! !

!BackgroundQueueProcessingJob methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    super initialize.
    queue := OrderedCollection new.
    queueAccessLock := RecursionLock new.

    "Modified: / 29-07-2011 / 10:22:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!BackgroundQueueProcessingJob methodsFor:'processing'!

process

    | item |

    [
        item := nil.
        queueAccessLock critical:[
            queue isEmpty ifFalse:[item := queue removeFirst]
        ].
        item isNil ifFalse:[
            self processItem: item
        ].
        item notNil.
    ] whileTrue

    "Created: / 28-04-2011 / 20:36:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 18-05-2011 / 23:30:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 03-08-2011 / 16:43:21 / cg"
!

processItem: item

    ^job value: item

    "Created: / 28-04-2011 / 20:36:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!BackgroundQueueProcessingJob class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic2/BackgroundQueueProcessingJob.st,v 1.8 2011-08-05 20:03:18 stefan Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/libbasic2/BackgroundQueueProcessingJob.st,v 1.8 2011-08-05 20:03:18 stefan Exp $'
!

version_SVN
    ^ 'Id'
! !
